/******************************************************************************/
/*                                                                            */
/*     globals.h   contains all globals as externals.                         */
/*                 This is the one include needed by nearly all functions     */
/*                 other than main.  It has everything except the function    */
/*                 definitions from in.h, fm.h and fc.h.                      */
/*                                                                            */
/******************************************************************************/
/* modified 4/92 by DBS@CWU for GRASS, X11, and multiuser support */
/*========================= standard includes ==============================*/

#include <stdio.h>
#include <termio.h>
#include "./define.h"
#include <sys/types.h>
#ifdef SV32
#include <sys/at_ansi.h>
#include <sys/kd.h>
#endif

/*========================= global variables ===============================*/

     /************************ busmse.h **********************************/
#define VGAMAP  ('V' << 8)
#define VGA_XHAIR	(VGAMAP | 10)
#define VGA_SETXHAIR	(VGAMAP | 13)
#define MOUSE_INIT	(('M' << 8) | 81)
#define GET_MOUSE	(('M' << 8) | 82)
extern int vga_is_open;
extern int mouse_is_open;
extern int tty_is_raw;
extern int vgafd;			/* /dev/vga file descriptor   */
extern int mousfd;			/* /dev/mouse file descriptor */
#ifndef REDHAT51
  extern struct args;
#endif

     /************************ control.h *********************************/
		/*    current line trace plus version number.   */
extern double ltp_version;   
         /*   config file contents  */
extern CONFIG config;
         /*   mode and exit flags   */
extern long batch_mode;    /* flag, 1 = batch mode = no prompts or graphics  */
extern long collecting_macro;/* flag, 1 = store commands, don't execute   */
extern char control_ban[]; /* string to display identifying calling program  */
extern long exit_macro;    /* flag, 1 = exit out from macro(s) to user input */
extern long exit_program;  /* flag, 1 = exit program now.                    */
extern long snap_input;    /* flag, 1 = snap input coords, 0 = don't snap    */
extern long input_mode;    /* input mode code,           see control.h       */
         /*   cursor_keys.h         */
extern char cursor_keys[][STRLIST];  /* the function produced by each keypress*/
         /*   macros list/control   */
extern char macros[][FNAME_LEN];/*an orderered list of all user macros  */
#ifdef MULTIUSER
extern char my_macros[][FNAME_LEN]; /* list of macros from users home dir */
#endif
extern int  macro_file;        /*  a file pointer to macro being collected */
extern long macro_level;       /*  current macro level being executed. */
        /*    menu control variables          */
extern char menu_item[MENUS][MENU_ITEMS][STRLIST];/* commands for all menus */
extern char menu_name[MENUS][STRLIST];      /* menu names in order       */
extern char menu_help[MENUS][STRLIST];      /* help list for curnt menus */
extern char menuset_name[STRLIST];          /* name of current menu set  */
extern long current_menu;                   /* number of current menu    */
extern char perm_item[6][STRLIST];          /* permanent menu items      */
         /*   stickup menu control  */
extern long stickup1_rows, stickup1_cols;   /* dimtns of stickup menu on dig1 */
extern long stickup2_rows, stickup2_cols;   /* dimtns of stickup menu on dig2 */
extern double stickup1reg[4][2];       /*  registration for stkup1 (dig cords)*/
extern double stickup2reg[4][2];       /*  registration for stkup2 (dig cords)*/
extern char stickup1[];                     /* contents of stickup menu dig1  */
extern char stickup2[];                     /* contents of stickup menu dig2  */
		/*	  malloc control variables                            */
extern char *malloc_ptr[2];    /*  pointers to the beginning & just past     */
                               /*  space reserved for internal malloc(_ltp)  */
extern char *mcontrol[100][2]; /*  program pointers for internal mallocs     */
		/*    undo control variables.   */
extern UNDO_STRUCT undo_array[]; /*  32 sets of mode & coords for rstr chages */
extern long undo_index;          /*  last changed undo_array item.            */

     /************************ errors.h **********************************/
extern char errors[][STRLIST]; /* a list of all error strings for user  */


     /************************ font.h ************************************/
extern unsigned char textchar[];       /*  bit maps for 256 characters       */


     /************************ func.h ************************************/
extern char uf_name[][15];  /*  array of fm names                         */
extern long (*fm_ptr[])();  /*  array of fm pointers                      */


     /************************ graphics.h ********************************/
extern long map_type;    /* bit flags, bit0=point, 1=line, 2=polygon     */
extern long anchorx;    /*  raster anchor x coordinate (rubber box/line) */
extern long anchory;    /*  raster anchor y coordinate (rubber box/line) */
extern long anchor2x;   /*  secondary anchor x coordinate (wedge)        */
extern long anchor2y;   /*  secondary anchor y coordinate (wedge)        */
extern long last_raster_x;    /* last raster x coord at input mode execution  */
extern long last_raster_y;    /* last raster y coord at input mode execution  */
extern long first_point;      /* first point needed for current input mode    */
extern long search_radius;    /* snap etc search radius (raster units)        */
extern long cursor_mode;      /* flag for cursor mode                         */
extern long colors[16];       /* values for 16 colors used on screen */
extern long graphics_flag;   /* flag indicating graphics state ( 1 = on )    */
extern long disp_coord_type;  /* coord type for display, -1 = none            */
extern int fdvga;             /* file pointer to the vga card                 */
extern long last_res_code;    /* code for last different resolution displayed */
extern long logo_on;          /* flag indicating initial LTP logo is on.      */
extern long resolution_code;  /* code for current displayed pixel resolution */
                       /*  raster coords of window for map display      */
extern long rwindowxmax,rwindowxmin,rwindowymax,rwindowymin;
extern char scratch_pad[][38400]; /*  pointer for saved graphics             */
extern long full_width_disp;  /*  flag for full width dispaly    */
extern long disp_raster;      /*  flag for raster display        */
extern long disp_centroid;    /*  flag for displaying raster     */
extern long disp_hglight;     /*  bit flags for vector objects highlighted */
extern long disp_normal;      /*  bit flags for vector objects normal display */
					/**    meaning of bit flags:  see graphics.h ***/
extern long disp_i1normal; /* bit flags for normal disp of import levels 32-63*/
extern long disp_i2normal; /* bit flags for normal disp of import levels 64-95*/
extern long disp_i1hglight;/* bit flags for hglght disp of import levels 32-63*/
extern long disp_i2hglight;/* bit flags for hglght disp of import levels 64-95*/

extern long horz_marker;    /*  screen x coord of horizontal (tile 8) marker */
extern long vert_marker;    /*  screen y coord of vertical (tile 9) marker   */
extern char text_buf[128][STRLIST];	/* for tile 4 strings. Allows scrolling   */
extern VIEW view[];         /* most recent x, y, and zoom factor for each res*/
extern char *vga_plane;       /* pointer to vga bitplane, mapped to.          */
                       /*  screen coords of window for map display      */
extern long windowxmax,windowxmin,windowymax,windowymin;


     /************************ input.h ***********************************/
extern char delim_buf[];          /*  delimeters for parsing input  */
extern long kb_nxtwr;             /*  relative ptr to next byte in kb_buf  */
extern unsigned char kb_buf[];    /*  buffer for cooked keyboard input     */
extern int kb_origflag;           /*  mode flags of stdin.                 */
extern char raw_buf[];            /*  holds command string for fj()        */
extern char atrbt_string[STRLIST]; /* current attribute for atrbt_dig mode */
extern long atrbt_otype;          /*  object type for atrbt_dig mode       */
extern long atrbt_action;         /*  action type for atrbt_dig mode       */
extern long elev_above;           /*  contour above */
extern long elev_below;           /*  contour below */
extern long context_elev;         /*  get elev from context flag */
extern long contour_interval;     /*  like it says (used for input)        */
extern long plus_plus;            /*  used for atrbt input                 */
extern long plus_length;          /*  used for atrbt input                 */
extern long entry_level;          /*  used for some input modes            */
extern double mile_post;          /*  value inserted when attrbting using $mp */
extern float circle_radius;       /*  circle radius when digitizing circles  */

extern TILECOORDS tcoords[];      /*  xmin,xmax,ymin,ymax for tiles    */
extern struct termio kb_tinfo;    /*  used by kbsetraw's ioctl calls       */
extern struct termio kb_tsave;    /*  original line settings of stdin      */
#ifdef SV32			/****  For AT&T only  ***/
extern struct kd_memloc kdm;      /*  used by graphics_on/off routines   */
extern strmap_t orig_fkeys;       /*  used by get_fkeys and reset_fkeys  */
extern scrnmap_t orig_chars;      /*  used to restore the original char set */
#endif
extern long which_buf;            /*  used by various input routines   */
	
/**  These vars are used by the graphics input  routines              **/
extern int  fd_plot;           /*  fd of open plotter device   */
extern int  fd_ginput;         /*  file descriptor of graphics input device */
extern long gr_dev;               /*  specifies the active graphics dev  */
extern GINFO ginfo;           /*  Cooked graphics input  information     */
extern DIGINFO diginfo;     /*  digitizer setup & interpretation information */
/**  These vars are used by and for the tiling functions       **/
extern long curr_menu;            /*  number of menu currently in tile 5   */
extern long curr_menu_item;       /*  menu item selected from tile 6       */

/**  This array contains function key string values           **/
#ifdef LTP_X11
#include <X11/Xlib.h>
extern KeySym fkey_vals[];              /* function key symbols */
#else
extern unsigned char fkey_vals[][8];	/*  function key strings  */
#endif

/**  These variables carry more general input information    **/
extern long linput_stats[];       /* current program status (integers)   */
extern double dinput_stats[];     /* current program status (doubles)    */
extern char sinput_stats[][STRLIST];/* current program status (strings)  */


     /************************ lists.h ***********************************/
extern long input_mode_limit;      /** highest mode currently used. **/
extern char input_modes[][15];
extern long atrbt_action_limit;
extern char atrbt_actions[][STRLIST];
extern char file_types[][STRLIST];


     /************************ map.h *************************************/
extern HEADER header;             /* contents of the header file           */
extern long state_plane_zone;     /* derived from header file              */
extern char level_names[96][STRLIST]; /* contents of level_names file.     */
extern char history[512][STRLIST];  /* contents of the history file.        */
extern long next_hist;    /*  ptr to the next (& oldest) history element    */
extern long job_stats[100];       /* contents of the job_stats file        */
extern long raster_wincrement;    /* number of 32-bit words in a raster row */
extern long raster_bincrement;    /* number of bytes in a raster row.       */
extern double corner[30][4][2]; /* contents of 1st part of the Register file */
extern double stpln_params[16]; /* contents of 2nd part of Register file     */
extern double lat_ref;          /* latitude reference for utm zone, dec. deg N*/
extern double lon_ref;          /* lon. ref. for utm (-180 to 180)            */
extern double map_scale;        /* Map scale from header file                 */
extern double tolerance;        /* rms comparison value for digitizer accuracy*/
extern double unit_factor[10][2];  /* pixel to other units conversion factors:
                                0 = raster pixels[0],  square raster pixels[1]
                                1 = map in.[0],  square inches[1]
                                2 = ground ft[0],  ground acres[1]
                                3 = ground mi[0],  square miles[1]
                                4 = map cm[0],  square cm[1]
                                5 = ground meters[0],  square meters[1]
                                6 = ground km[0],  square km[1]
                                7 = - unused -
                                8 = - unused -
                                9 = - unused -            *********/

     /************************ paths.h ***********************************/
extern char map_name[];     /*   the map file directory name (without path) */
extern char ltp_path[];     /*   the path of the ltp directory              */
extern char map_path[];     /*   the path of the current map                */
extern char temp_map_path[512];/*path of the temp file for the current map    */
extern char maps_path[];    /*   the path of the maps directory             */
extern char menus_path[];   /*   the path of the menus directory            */
extern char digmenus_path[];/*   the path of the digmenus directory         */
extern char import_path[];  /*   the path of the default import directory   */
extern char export_path[];  /*   the path of the defulat export directory   */
extern char macros_path[];  /*   the path of the macros directory           */
#ifdef MULTIUSER
extern char home_path[];    /*   the path of the user's home directory      */
#endif
#ifdef GRASS
extern char grass_path[];   /* the path to the user's GRASS location/mapset */
#endif

extern int objects_fd;     /*   file descriptor of objects file           */
extern int coord_list_fd;  /*   file descriptor of coord_list file        */
extern int seg_list_fd;    /*   file descriptor of seg_list file          */
extern int attributes_fd;  /*   file descriptor of attributes file        */
extern int errors_fd;      /*   file descriptor of errors file            */
extern int index_list_fd;  /*   file descriptor of index_list file        */
extern int v_attributes_fd;/*   file descriptor of v_attributes file      */
extern int v_objects_fd;   /*   file descriptor of v_objects file         */
extern int v_coordlist_fd; /*   file descriptor of v_coordlist file       */

extern unsigned long **lraster;  /*  pointer*pointer array, raster as longs  */
extern unsigned char **craster;  /*  pointer*pointer array, raster as chars  */
extern MALLOC_STRUCT rasterm;    /*  structure controlling raster malloc */
extern unsigned char mini_raster[400][50];/*  mini-raster array               */
extern MALLOC_STRUCT objectsm;   /*  structure controlling objects malloc */
extern MALLOC_STRUCT coord_listm;/*  structure controlling coord_list malloc */
extern MALLOC_STRUCT seg_listm;  /*  structure controlling seg_list malloc */
extern MALLOC_STRUCT attributesm;/*  structure controlling attributes malloc */
extern MALLOC_STRUCT errorsm;    /*  structure controlling errors malloc */
extern long **index_array;       /*  pointer*pointer array of the index array */
extern MALLOC_STRUCT index_arraym;/* structure controlling index_array malloc */
extern MALLOC_STRUCT index_listm;/*  structure controlling index_list malloc */
extern MALLOC_STRUCT v_attributesm;/*structure controlling v_attributes malloc*/
extern MALLOC_STRUCT v_objectsm; /*  structure controlling v_objects malloc */
extern MALLOC_STRUCT v_coordlistm;/* structure controlling v_coordlist malloc */

extern char *node_lookup_ptr; /*  pointer to malloc'd space for node_row array*/
extern long *node_lookup;     /*  pointer for simulated (lookup nodes) array  */

     /************************ pens.h ************************************/
extern char pens[];            /* Pen number for each set, in order          */


     /************************ prompts.h *********************************/
extern char *prompt[];


     /************************ set.h *************************************/
extern long set_criteria[];      /*        no. of criteria of a given type    */
extern long critadj[];           /*  specifies set numbers for adj polygons   */
extern char critarange[10][2][STRLIST]; /*  10 crit_arange strings */
extern double critarea[10][3];    /*  10 sets of crit_area ranges & codes*/
extern char critastrg[10][STRLIST];   /*  10 crit_astrg coded strings     */
extern double critlength[10][3];  /*  10 sets of crit_lenght ranges/codes*/
 /*  critnewatr      - - not needed, set_criteria[5] is all that's needed. */
extern long critpsides[10][2];    /*  10 pairs of psides ranges.         */
extern long critrecord[10][2];    /*  10 pairs of record number ranges.  */
extern long critserial[10][2];    /*  10 pairs of serial number ranges.  */
extern long critslen[10][2];      /*  10 pairs of string length ranges.  */
extern long critsubel[10];         /*  10 set numbers                     */
extern long critsuperel[10];       /*  10 set numbers                     */
extern long crittype[10];          /*  10 type codes                      */


     /************************ tables.h **********************************/
extern long int tlocale[];  /* keys for all 256 possible Locale values        */


     /************************ tiles.h ***********************************/
extern long t4_indx;		/*  index into text_buf for tile 4            */
