/******************************************************************************/
/*                                                                            */
/*     tables.h    include file.                                              */
/*                 Raster lookup tables (common variable)                     */
/*                                                                            */
/******************************************************************************/
long int tlocale[256];  /* keys for all 256 possible Locale values        */
    /*  This is a lookup table which holds information about the various pixel*/
    /* arrangements which can surround a given pixel.  The 8 pixels           */
    /* surrounding a given pixel are first coded into a integer variable      */
    /* by setting bits on or off according to the following map:              */
    /*                                                                        */
    /*                                                 7 0 1                  */
    /*                                                 6 X 2                  */
    /*                                                 5 4 3                  */
    /*                                                                        */
    /*                                                                        */
    /*  Thus the pixel arrangement around X:           0 1 1                  */
    /*                                                 1 X 0   (1) = on       */
    /*                                                 0 1 1   (0) = off      */
    /*                                                                        */
    /*  Would code into the bits:  01011011  which is the integer 91.  (Bit 0 */
    /* is on the right.)  I call this variable 'Locale'.  Using this method,  */
    /* the possible pixel arrangements translate into the integers 0 through  */
    /* 255.  For each integer, there is one element in the array Tlocale.     */
    /* Each element of the array consists of bit flags & imbedded integers as */
    /* follows:					                              */
    /*                                                                        */
    /*   bits   meaning                                                       */
    /*    0-7  The 'adjusted' Locale variable.  The adjustment is made by     */
    /*         turning off all corner bits which have an 'on' side bit        */
    /*         beside them.  This represents the directions in which a        */
    /*         pixel arrangement should be traced.                            */
    /*     8   bit flag, 1 indicates isolated point (0 branch node)           */
    /*     9   bit flag, 1 indicates a spur end     (1 branch node)           */
    /*    10   bit flag, 1 indicates a segment   (or 2 branch node)           */
    /*    11   bit flag, 1 indicates a 3 branch node                          */
    /*    12   bit flag, 1 indicates a 4 branch node                          */
    /*    13   bit flag, 1 indicates that a node (any kind) at this pixel     */
    /*    14   bit flag, 1 indicates that thinning is complete at this pixel  */
    /*    15   bit flag, 1 indicates remove pixel for type 0 thinning         */
    /*         (nondestructive, leave spur ends, remove square corners and    */
    /*         stairsteps.)                                                   */
    /*    16   bit flag, 1 indicates remove pixel for type 1 thinning         */
    /*         (nondestructive, leave spur ends, leave square corners and     */
    /*         stairsteps.)                                                   */
    /*    17   bit flag, 1 indicates remove pixel for type 2 thinning         */
    /*         (DESTRUCTIVE, remove all pixels which are not part of a solid  */
    /*         interior.)  This flag indicates all pixels which are not       */
    /*         part of a solid interior (with a 1), and conversely pixels     */
    /*         which are part of a solid interior (with a 0).                 */
    /*    18   bit flag, 1 indicates that this pixel configuration in the     */
    /*         unthinned (old) raster requires an additional check (due to    */
    /*         directionality of thinning).  Check the 'new' (thinned above)  */
    /*         pixel locale against the value below, and thin if it matches.  */
    /*         (Types 0 and 1 thinning only)                                  */
    /*   19-26 Test 'new' locale to be used in conjunction with 18 above.  If */
    /*         18 is flagged and there is a match between this and the 'new'  */
    /*         locale, then the item should be type 0 or type 1 thinned.      */
    /*         Not used on type 2 (destructive) thinning.                     */
    /*  27-28  unused                                                         */
    /*  29-31  a 3 bit integer (unsigned) indicating the number of branches   */
    /*         from the center pixel location.  For Line Trace use this will  */
    /*         always be a number from 0 to 4 since lines are not traced to   */
    /*         corners if the side pixel is on.                               */
    /*                                                                        */
    /*  Note:  The adjusted locale variable (bits 7-0 of the lookup table) is */
    /*  adjusted by turning off bits which represent corner pixels if an      */
    /*  adjacent side pixel is on.  For example:                              */
    /*                                                                        */
    /*  1 1 1               0 1 0                                             */
    /*  1 X 1  adjusts to   1 X 1  (locale=255  becomes node_type=85)         */
    /*  1 1 1               0 1 0                                             */
    /*                                                                        */
    /*  1 1 1               0 1 0                                             */
    /*  0 X 0  adjusts to   0 X 0  (locale=171  becomes node_type=41)         */
    /*  1 0 1               1 0 1                                             */
    /*                                                                        */
    /*  (The branches are counted after adjustment.  The thinning information */
    /*  is calculated on the unadjusted locale.)                              */
